package com.uclbrt.myapplication;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.support.design.widget.NavigationView;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.support.v4.view.GravityCompat;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.text.InputType;
import android.text.TextUtils;
import android.view.MenuItem;
import android.widget.EditText;
import android.widget.Toast;
import com.uclbrt.QRMasterBTSDK.bluetooth.lock.UclbrtBleLockErrorCode;
import com.uclbrt.QRMasterBTSDK.exception.BleStrException;
import com.uclbrt.QRMasterBTSDK.fetcher.KeyFetcher;
import com.uclbrt.QRMasterBTSDK.fetcher.KeyGenerator;
import com.uclbrt.QRMasterBTSDK.fetcher.KeyInvalid;
import com.uclbrt.QRMasterBTSDK.fetcher.listener.IOpenDoorListener;
import com.uclbrt.QRMasterBTSDK.fetcher.listener.KeyFetcherListener;
import com.uclbrt.QRMasterBTSDK.fetcher.listener.KeyGeneratorListener;
import com.uclbrt.QRMasterBTSDK.fetcher.listener.KeyInvalidListener;
import com.uclbrt.QRMasterBTSDK.utils.ToastUtils;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

public class MainActivity extends AppCompatActivity
    implements NavigationView.OnNavigationItemSelectedListener {

  private KeyGenerator mKeyGenerator;
  private KeyInvalid mKeyInvalid;
  private KeyFetcher keyFetcher;
  private String mCardNo;
  private ProgressDialog mProgressDialog;

  private SimpleDateFormat format = new SimpleDateFormat("yyMMddHHmm", Locale.getDefault());
  // 伪造数据
  String communityNo = "1316879946";
  String mobile = "10000000000"; // 手机号
  String buildNo = "001";
  String floorNo = "001";
  String roomNo = "101"; // 锁掌柜房间号
  long startTime = System.currentTimeMillis(); // unix时间戳，当前时间作为有效时间
  String startTimeStr = format.format(new Date(startTime));
  long endTime = startTime + 3600000; // unix时间戳，10分钟后作为结束时间
  String endTimeStr = format.format(new Date(endTime));
  String bleStr = "";

  @Override protected void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.activity_main);
    Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
    setSupportActionBar(toolbar);

    DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
    ActionBarDrawerToggle toggle =
        new ActionBarDrawerToggle(this, drawer, toolbar, R.string.navigation_drawer_open,
            R.string.navigation_drawer_close);
    drawer.addDrawerListener(toggle);
    toggle.syncState();

    NavigationView navigationView = (NavigationView) findViewById(R.id.nav_view);
    navigationView.setNavigationItemSelectedListener(this);

    mProgressDialog = new ProgressDialog(this);
    mProgressDialog.hide();

    // 生成房卡类
    mKeyGenerator = new KeyGenerator(Config.UCLBRT_SID, Config.UCLBRT_TOKEN);
    // 监听生成房卡类回调
    mKeyGenerator.setListener(new KeyGeneratorListener() {
      @Override public void onSuccess(String cardNo) {
        // TODO 后台线程返回结果
        // TODO 运行在main线程
        hideLoading();
        mCardNo = cardNo;
        ToastUtils.show(MainActivity.this, "生成房卡 " + cardNo);
      }

      @Override public void onError(String message, int status) {
        // TODO 后台线程返回结果
        // TODO 运行在main线程
        hideLoading();
        ToastUtils.show(MainActivity.this, message);
      }
    });

    // 失效房卡类
    mKeyInvalid = new KeyInvalid(Config.UCLBRT_SID, Config.UCLBRT_TOKEN);
    // 监听
    mKeyInvalid.setListener(new KeyInvalidListener() {
      @Override public void onSuccess() {
        // TODO 后台线程返回结果
        // TODO 运行在main线程
        hideLoading();
        ToastUtils.show(MainActivity.this, "房卡已失效");
        mCardNo = "";
      }

      @Override public void onError(String message, int status) {
        // TODO 后台线程返回结果
        // TODO 运行在main线程
        hideLoading();
        ToastUtils.show(MainActivity.this, message);
      }
    });

    // 获取钥匙
    keyFetcher = new KeyFetcher(this, Config.UCLBRT_SID, Config.UCLBRT_TOKEN);

    keyFetcher.onCreate(this);

    // 设置蓝牙的连接超时时间，单位毫秒，默认30000毫秒
    keyFetcher.setBleScanTimeout(30000);
    // 设置蓝牙发送数据的超时时间，单位毫秒，默认10000毫秒
    keyFetcher.setBleSendDataTimeout(10000);

    // 监听获取钥匙回调
    keyFetcher.setKeyFetcherListener(new KeyFetcherListener() {
      @Override public void onSuccess(Fragment fragment) {
        hideLoading();

        // 需要先判断Fragment是否为空
        if (fragment != null) {
          // 加载qrb开门界面
          FragmentManager manager = getSupportFragmentManager();
          FragmentTransaction transaction = manager.beginTransaction();
          transaction.replace(R.id.content, fragment);
          transaction.show(fragment);
          // 可选 添加获取钥匙错误显示的Fragment
          transaction.commit();
        } else {
          showLoading();
          // 共享版，没有界面，直接连接蓝牙开门
        }
      }

      @Override public void onError(String message, int status) {
        ToastUtils.show(MainActivity.this, message);
        hideLoading();
      }
    });
    // 设置开门监听
    keyFetcher.setOpenDoorListener(new IOpenDoorListener() {
      @Override public void onOpenDoorSuccess(String msg) {
        // TODO 后台线程返回结果
        // TODO 运行在main线程
        hideLoading();
        ToastUtils.show(MainActivity.this, msg);
      }

      @Override public void onOpenDoorFailure(int errorCode) {
        // TODO 后台线程返回结果
        // TODO 运行在main线程
        hideLoading();
        ToastUtils.show(MainActivity.this,
            UclbrtBleLockErrorCode.getErrorStr(MainActivity.this, errorCode));
        // TODO 使用errorCode请参照文档 (2.6 IOpenDoorListener监听开门失败返回的errorCode对照表)
      }
    });
  }

  @Override protected void onDestroy() {
    super.onDestroy();

    keyFetcher.onDestroy(this);
  }

  @Override public void onBackPressed() {
    DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
    if (drawer.isDrawerOpen(GravityCompat.START)) {
      drawer.closeDrawer(GravityCompat.START);
    } else {
      super.onBackPressed();
    }
  }

  @SuppressWarnings("StatementWithEmptyBody") @Override
  public boolean onNavigationItemSelected(MenuItem item) {
    int id = item.getItemId();

    if (id == R.id.nav_create) {
      showLoading();
      startTime = System.currentTimeMillis(); // unix时间戳，当前时间作为有效时间
      endTime = startTime + 3600000; // unix时间戳，10分钟后作为结束时间
      startTimeStr = format.format(new Date(startTime));
      endTimeStr = format.format(new Date(endTime));
      mKeyGenerator.createCard(mobile, communityNo, buildNo, floorNo, roomNo, startTimeStr,
          endTimeStr, 86);

      // 刷新开门状态
      //mBleKeyFragment.refreshStatus();
    } else if (id == R.id.nav_cancel) {
      showLoading();
      mKeyInvalid.cancelCard(mCardNo);
    } else if (id == R.id.nav_show) {
      if (TextUtils.isEmpty(mCardNo)) {
        Toast.makeText(this, "获取房卡前，请先调用createCard创建房卡，createCard的参数须与getCardByRoom参数一致",
            Toast.LENGTH_LONG).show();
        return true;
      }
      showLoading();
      // 获取钥匙
      // 通过楼栋房间信息获取钥匙
      keyFetcher.getCardByRoom(communityNo, buildNo, floorNo, roomNo, startTimeStr, endTimeStr,
          "86", mobile);
    } else if (id == R.id.nav_show2) {
      if (TextUtils.isEmpty(mCardNo)) {
        ToastUtils.show(this, "获取房卡前，请先调用createCard创建房卡");
        return true;
      }
      showLoading();
      // 获取钥匙
      // 使用CardNo来获取钥匙
      keyFetcher.getCardByCardNo(communityNo, mCardNo, "86", mobile);
    } else if (id == R.id.nav_show3) {
      showLoading();
      try {
        keyFetcher.openDoor(this, bleStr);
      } catch (BleStrException e) {
        hideLoading();
        ToastUtils.show(this, "输入的bleStr格式不正确");
        e.printStackTrace();
      }
    } else if (id == R.id.nav_set_ble_str) {
      setBleStrDialog();
    } else if (id == R.id.nav_ble_scan_timeout) {
      setBleScanTimeoutDialog();
    } else if (id == R.id.nav_ble_send_data_timeout) {
      setBleSendDataTimeoutDialog();
    }

    DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
    drawer.closeDrawer(GravityCompat.START);
    return true;
  }

  private void showLoading() {
    mProgressDialog.show();
  }

  private void hideLoading() {
    mProgressDialog.hide();
  }

  private void setBleStrDialog() {
    AlertDialog.Builder builder = new AlertDialog.Builder(this);
    final EditText editText = new EditText(this);
    builder.setView(editText)
        .setTitle("input ble str")
        .setPositiveButton("OK", new DialogInterface.OnClickListener() {
          @Override public void onClick(DialogInterface dialog, int which) {
            bleStr = editText.getText().toString();
            dialog.dismiss();
          }
        })
        .show();
  }

  private void setBleScanTimeoutDialog() {
    AlertDialog.Builder builder = new AlertDialog.Builder(this);
    final EditText editText = new EditText(this);
    editText.setText(String.valueOf(keyFetcher.getBleScanTimeout()));
    editText.setInputType(InputType.TYPE_CLASS_NUMBER);
    builder.setView(editText)
        .setTitle("input ble scan timeout")
        .setPositiveButton("OK", new DialogInterface.OnClickListener() {
          @Override public void onClick(DialogInterface dialog, int which) {
            if (TextUtils.isEmpty(editText.getText().toString())) {
              dialog.dismiss();
              return;
            }
            long bleScanTimeout;
            try {
              bleScanTimeout = Long.parseLong(editText.getText().toString());
            } catch (Exception e) {
              dialog.dismiss();
              return;
            }
            keyFetcher.setBleScanTimeout(bleScanTimeout);
            dialog.dismiss();
          }
        })
        .setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
          @Override public void onClick(DialogInterface dialog, int which) {
            dialog.dismiss();
          }
        })
        .show();
  }

  private void setBleSendDataTimeoutDialog() {
    AlertDialog.Builder builder = new AlertDialog.Builder(this);
    final EditText editText = new EditText(this);
    editText.setText(String.valueOf(keyFetcher.getBleSendDataTimeout()));
    editText.setInputType(InputType.TYPE_CLASS_NUMBER);
    builder.setView(editText)
        .setTitle("input ble send data timeout")
        .setPositiveButton("OK", new DialogInterface.OnClickListener() {
          @Override public void onClick(DialogInterface dialog, int which) {
            if (TextUtils.isEmpty(editText.getText().toString())) {
              dialog.dismiss();
              return;
            }
            long bleSendDataTimeout;
            try {
              bleSendDataTimeout = Long.parseLong(editText.getText().toString());
            } catch (Exception e) {
              dialog.dismiss();
              return;
            }
            keyFetcher.setBleSendDataTimeout(bleSendDataTimeout);
            dialog.dismiss();
          }
        })
        .setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
          @Override public void onClick(DialogInterface dialog, int which) {
            dialog.dismiss();
          }
        })
        .show();
  }
}
